Graphics3D 800,600,32,2

Include "texture_splatting.bb"


; Player movement speed
Global PlayerSpeed#=0.5


; Frametweening
Global GameSpeed%=60
Global Screenwidth%=GraphicsWidth()
Global Screenheight%=GraphicsHeight()
Global FramePeriod%=1000/GameSpeed
Global FrameTime%=MilliSecs()-FramePeriod


; move variables
Global Camera,SpeedMultiplicator%,DeltaTimeOld


; world
Global Pivot%,Light%


; central pivot
Pivot=CreatePivot()


; Camera
Camera=CreateCamera()
PositionEntity Camera,-32,64,-64
CameraZoom Camera,1
CameraRange Camera,0.1,256
CameraFogMode Camera,True
CameraFogRange Camera,96,192
CameraFogColor Camera,155,174,206

CameraClsColor Camera,155,174,206
PointEntity Camera,Pivot


; Light
Light=CreateLight(2)
PositionEntity Light,10000,10000,10000
LightRange Light,20000
AmbientLight 16,16,16



; Load maps
hm=LoadImage("maps/hm.png")	; Heightmap
lm=LoadImage("maps/lm.png")	; Lightmap
cm=LoadImage("maps/cm.png")	; Colormap


; create mesh
mesh=CreateMesh() : h#=24 ; height multiplier


; create Meshterrain (returns surface indexes) - the order must be descending top>down with the base layer on very top
grass	= LoadMeshTerrain(mesh,hm,h,cm,1.0,LoadImage("maps/alpha_grass.png"),False,False,False,True)
smile	= LoadMeshTerrain(mesh,hm,h,False,1.0,LoadImage("maps/alpha_smile.png"),192,192,128)
road	= LoadMeshTerrain(mesh,hm,h,False,1.0,LoadImage("maps/alpha_road.png"),0,255,0)
snow	= LoadMeshTerrain(mesh,hm,h,False,1.0,LoadImage("maps/alpha_snow.png"),255,255,255)
rock	= LoadMeshTerrain(mesh,hm,h,lm,1.0,LoadImage("maps/alpha_rock.png"))


; apply textures to surface indexes
ApplyTexture("textures/grass.jpg"		,0,1.0/ 32,1.0/ 32,5,False,grass	,2)
ApplyTexture("textures/crystal.jpg"		,0,1.0/  4,1.0/  4,1,False,smile	,2+32)
ApplyTexture("textures/road.jpg"		,0,1.0/128,1.0/128,5,False,road		,2+32)
ApplyTexture("textures/rock.jpg"		,0,1.0/ 32,1.0/ 32,2,False,rock		,2+32)
ApplyTexture("textures/snow.jpg"		,0,1.0/ 16,1.0/ 16,2,False,snow		,2+32)


; Apply detail texture to meshes
detail=LoadTexture("textures/detail.jpg") : TextureBlend detail,5 : ScaleTexture detail,1.0/64,1.0/64
EntityTexture mesh,detail,0,1


; centered pointer
MoveMouse GraphicsWidth()/2,GraphicsHeight()/2


; Main loop
While Not KeyHit(1)
	
	Cls
	
	; Frametween calculation
	Local FrameElapsed%,FrameTicks%,FrameTween#,t%
	Repeat FrameElapsed=MilliSecs()-FrameTime Until FrameElapsed
	FrameTicks=FrameElapsed/FramePeriod
	FrameTween=Float(FrameElapsed Mod FramePeriod)/Float(FramePeriod)
	
	; Frametween loop
	For t=1 To FrameTicks
		
		; Frametween Captureworld
		FrameTime=FrameTime+FramePeriod : If t=FrameTicks Then CaptureWorld
	
		; Space = Wireframe on/off
		If KeyHit(57) Then wf%=1-wf : WireFrame wf
		
		; Movement
		FreeCam()
		
		UpdateWorld
		
	Next
	
	; Frametween Render
	RenderWorld FrameTween
	
	Text 0,0,TrisRendered()
	
	; Bufferflip
	Flip 0
	
Wend

End


; customized Loadtexture Function
Function ApplyTexture(filename$,flags%,uscale#=1.0,vscale#=1.0,blendmode%=False,mesh%,surf%=False,fx%=False)
	
	Local tex%=LoadTexture(filename$,flags)
	ScaleTexture tex,uscale,vscale
	If blendmode Then TextureBlend tex,blendmode
	
	Local brush%=CreateBrush(255,255,255)
	If tex Then BrushTexture brush,tex
	If fx Then BrushFX brush,fx
	
	If surf Then
		PaintSurface surf,brush
	Else 
		surf=CreateSurface(mesh)
		PaintSurface surf,brush
	EndIf
	
	FreeBrush brush
	
	Return tex
	
End Function


; Freecam Movement
Function FreeCam(maxpitch#=85.0,rotspeed#=16.666,rotfloat#=8.0)
	
	Local movex#,movez#,dx#,dy#,dk#,dt%,t%
	Local movespeed#,pitch#
	
	SpeedMultiplicator=1
	
	; LMB/RMB = 2x faster
	If MouseDown(1) Or MouseDown(2) Then SpeedMultiplicator=2
	
	; Arrows = Move
	movex=KeyDown(205)-KeyDown(203)
	movez=KeyDown(200)-KeyDown(208)
	
	; Movespeed
	movespeed=PlayerSpeed*SpeedMultiplicator
	
	; smooth movement
	t=MilliSecs() : dt=t-DeltaTimeOld : DeltaTimeOld=t : dk=Float(dt)/rotspeed
	dx=(Screenwidth/2-MouseX())*0.01*dk : dy=(Screenheight/2-MouseY())*0.01*dk
	TurnEntity Camera,-dy,dx*0.1*dk*rotfloat,0
	
	; limit pitch
	pitch=EntityPitch(Camera,1) : If pitch>maxpitch Then pitch=maxpitch Else If pitch<-maxpitch Then pitch=-maxpitch
	
	; rotate and move
	RotateEntity Camera,pitch,EntityYaw(Camera,1),0,1	
	MoveEntity Camera,movex*movespeed,0,movez*movespeed
	
End Function
;~IDEal Editor Parameters:
;~C#Blitz3D